"""
Copyright (c) 2008-2021 synodriver <synodriver@gmail.com>
"""
from pylzf.backends.cffi._lzf import ffi, lib


def compress(data: bytes, outlen: int = None) -> bytes:
    c_data = ffi.from_buffer(data)
    if outlen is None:
        outlen = ((len(data) * 33) >> 5) + 1
    out = ffi.new(f"char[{outlen}]")
    ret = lib.lzf_compress(
        ffi.cast("void *", c_data), len(data), ffi.cast("void *", out), outlen
    )
    if ret == 0:
        raise ValueError
    return ffi.unpack(out, ret)


def decompress(data: bytes, outlen: int) -> bytes:
    c_data = ffi.from_buffer(data)
    out = ffi.new(f"char[{outlen}]")
    ret = lib.lzf_decompress(
        ffi.cast("void *", c_data), len(data), ffi.cast("void *", out), outlen
    )
    if ret == 0:
        raise ValueError
    return ffi.unpack(out, ret)


def compress_into(data: bytes, out: bytearray) -> int:
    in_data = ffi.from_buffer(data)
    out_data = ffi.from_buffer(out)
    ret = lib.lzf_compress(
        ffi.cast("void *", in_data), len(data), ffi.cast("void *", out_data), len(out)
    )
    if ret == 0:
        raise ValueError
    return ret


def decompress_into(data: bytes, out: bytearray) -> int:
    in_data = ffi.from_buffer(data)
    out_data = ffi.from_buffer(out)
    ret = lib.lzf_decompress(
        ffi.cast("void *", in_data), len(data), ffi.cast("void *", out_data), len(out)
    )
    if ret == 0:
        raise ValueError
    return ret


LZF_VERSION = lib.version()



# cython: language_level=3
# cython: cdivision=True
from cpython.bytes cimport PyBytes_AS_STRING, PyBytes_FromStringAndSize
from libc.stdint cimport uint8_t

from pylzf.backends.cython.lzf cimport (LZF_VERSION_C, lzf_compress,
                                        lzf_decompress)


cpdef inline bytes compress(const uint8_t[::1] data, Py_ssize_t output_size = 0):
    cdef Py_ssize_t input_size = data.shape[0]
    if output_size == 0:
        output_size = ((input_size * 33) >> 5 ) + 1
    cdef bytes out = PyBytes_FromStringAndSize(NULL, output_size)
    if <void*>out == NULL:
        raise MemoryError
    cdef unsigned int ret
    cdef void* out_ptr = <void*> PyBytes_AS_STRING(out)
    with nogil:
        ret = lzf_compress(<void*>&data[0],<unsigned int> input_size, out_ptr, <unsigned int> output_size)
    if ret == 0:
        raise ValueError
    return out[:ret]

cpdef inline bytes decompress(const uint8_t[::1] data, unsigned int outlen):
    cdef Py_ssize_t input_size = data.shape[0]
    cdef bytes out = PyBytes_FromStringAndSize(NULL, outlen)
    if <void*>out == NULL:
        raise MemoryError
    cdef unsigned int ret
    cdef void* out_ptr = <void*> PyBytes_AS_STRING(out)
    with nogil:
        ret = lzf_decompress(<void*>&data[0],<unsigned int> input_size, out_ptr, outlen)
    if ret == 0:
        raise ValueError
    return out[:ret]

cpdef inline unsigned int compress_into(const uint8_t[::1] data, uint8_t[::1] out) except? 0:
    cdef unsigned int ret
    with nogil:
        ret = lzf_compress(<void *> &data[0], <unsigned int> data.shape[0],
                                            <void *> &out[0], <unsigned int> out.shape[0])
    if ret == 0:
        raise ValueError
    return ret

cpdef inline unsigned int decompress_into(const uint8_t[::1] data, uint8_t[::1] out) except? 0:
    cdef unsigned int ret
    with nogil:
        ret = lzf_decompress(<void *> &data[0], <unsigned int> data.shape[0],
                                            <void *> &out[0], <unsigned int> out.shape[0])
    if ret == 0:
        raise ValueError
    return ret

LZF_VERSION = LZF_VERSION_C



import glob
import sys

from cffi import FFI

ffibuilder = FFI()
ffibuilder.cdef(
    """
unsigned int 
lzf_compress (const void *const in_data,  unsigned int in_len,
              void             *out_data, unsigned int out_len);
              
unsigned int 
lzf_decompress (const void *const in_data,  unsigned int in_len,
                void             *out_data, unsigned int out_len);
int version();
    """
)

source = """
#include "lzf.h"
#include "lzfP.h"

int version()
{
    return LZF_VERSION;
}
"""

ffibuilder.set_source(
    "pylzf.backends.cffi._lzf",
    source,
    sources=["./liblzf/lzf_c.c", "./liblzf/lzf_d.c"],
    include_dirs=["./liblzf"],
)

if __name__ == "__main__":
    ffibuilder.compile()



# cython: language_level=3
# cython: cdivision=True
cdef extern from "lzf.h" nogil:
    unsigned int lzf_compress(void *in_data,  unsigned int in_len, void *out_data, unsigned int out_len)
    unsigned int lzf_decompress(void *in_data,  unsigned int in_len, void  *out_data, unsigned int out_len)
    int LZF_VERSION_C "LZF_VERSION"